@@ -10,12 +10,12 @@ All rights reserved.
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 		Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
-		Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the 
+		Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the
 		documentation and/or other materials provided with the distribution.
 		Neither the name of AirSpy nor the names of its contributors may be used to endorse or promote products derived from this software
 		without specific prior written permission.
 
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
+THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
@@ -61,6 +61,9 @@ typedef int bool;
 #define USB_PRODUCT_ID (2)
 #define STR_DESCRIPTOR_SIZE (250)
 
+#define MIN_SAMPLERATE_BY_VALUE (1000000)
+#define SAMPLE_TYPE_IS_IQ(x) ((x) == AIRSPY_SAMPLE_FLOAT32_IQ || (x) == AIRSPY_SAMPLE_INT16_IQ)
+
 typedef struct {
 	uint32_t freq_hz;
 } set_freq_params_t;
@@ -74,16 +77,19 @@ typedef struct airspy_device
 	volatile bool streaming;
 	volatile bool stop_requested;
 	pthread_t transfer_thread;
-	pthread_t conversion_thread;
-	pthread_cond_t conversion_cv;
-	pthread_mutex_t conversion_mp;
+	pthread_t consumer_thread;
+	pthread_cond_t consumer_cv;
+	pthread_mutex_t consumer_mp;
+	uint32_t supported_samplerate_count;
+	uint32_t *supported_samplerates;
 	uint32_t transfer_count;
 	uint32_t buffer_size;
-	uint32_t total_dropped_samples;	
+	uint32_t dropped_buffers;
+	uint32_t dropped_buffers_queue[RAW_BUFFER_COUNT];
 	uint16_t *received_samples_queue[RAW_BUFFER_COUNT];
 	volatile int received_samples_queue_head;
 	volatile int received_samples_queue_tail;
-	volatile bool converter_is_waiting;
+	volatile int received_buffer_count;
 	void *output_buffer;
 	uint16_t *unpacked_samples;
 	bool packing_enabled;
@@ -97,7 +103,7 @@ static const uint16_t airspy_usb_vid = 0x1d50;
 static const uint16_t airspy_usb_pid = 0x60a1;
 
 #define STR_PRODUCT_AIRSPY_SIZE (6)
-static const char str_product_airspy[STR_PRODUCT_AIRSPY_SIZE] = 
+static const char str_product_airspy[STR_PRODUCT_AIRSPY_SIZE] =
 { 'A', 'I', 'R', 'S', 'P', 'Y' };
 
 #define STR_PREFIX_SERIAL_AIRSPY_SIZE (10)
@@ -119,17 +125,18 @@ static int cancel_transfers(airspy_device_t* device)
 {
 	uint32_t transfer_index;
 
-	if( device->transfers != NULL )
+	if (device->transfers != NULL)
 	{
-		for(transfer_index=0; transfer_index<device->transfer_count; transfer_index++)
+		for (transfer_index = 0; transfer_index<device->transfer_count; transfer_index++)
 		{
-			if( device->transfers[transfer_index] != NULL )
+			if (device->transfers[transfer_index] != NULL)
 			{
 				libusb_cancel_transfer(device->transfers[transfer_index]);
 			}
 		}
 		return AIRSPY_SUCCESS;
-	} else {
+	}
+	else {
 		return AIRSPY_ERROR_OTHER;
 	}
 }
@@ -142,10 +149,11 @@ static int free_transfers(airspy_device_t* device)
 	if (device->transfers != NULL)
 	{
 		// libusb_close() should free all transfers referenced from this array.
-		for(transfer_index=0; transfer_index < device->transfer_count; transfer_index++)
+		for (transfer_index = 0; transfer_index < device->transfer_count; transfer_index++)
 		{
-			if( device->transfers[transfer_index] != NULL )
+			if (device->transfers[transfer_index] != NULL)
 			{
+				free(device->transfers[transfer_index]->buffer);
 				libusb_free_transfer(device->transfers[transfer_index]);
 				device->transfers[transfer_index] = NULL;
 			}
@@ -158,7 +166,7 @@ static int free_transfers(airspy_device_t* device)
 			free(device->output_buffer);
 			device->output_buffer = NULL;
 		}
-		
+
 		if (device->unpacked_samples != NULL)
 		{
 			free(device->unpacked_samples);
@@ -184,7 +192,7 @@ static int allocate_transfers(airspy_device_t* const device)
 	size_t sample_count;
 	uint32_t transfer_index;
 
-	if( device->transfers == NULL )
+	if (device->transfers == NULL)
 	{
 		for (i = 0; i < RAW_BUFFER_COUNT; i++)
 		{
@@ -205,13 +213,13 @@ static int allocate_transfers(airspy_device_t* const device)
 		{
 			sample_count = device->buffer_size / 2;
 		}
-		
-		device->output_buffer = (float *) malloc(sample_count * sizeof(float));
+
+		device->output_buffer = (float *)malloc(sample_count * sizeof(float));
 		if (device->output_buffer == NULL)
 		{
 			return AIRSPY_ERROR_NO_MEM;
 		}
-		
+
 		if (device->packing_enabled)
 		{
 			device->unpacked_samples = (uint16_t*)malloc(sample_count * sizeof(uint16_t));
@@ -222,31 +230,31 @@ static int allocate_transfers(airspy_device_t* const device)
 		}
 
 		device->transfers = (struct libusb_transfer**) calloc(device->transfer_count, sizeof(struct libusb_transfer));
-		if( device->transfers == NULL )
+		if (device->transfers == NULL)
 		{
 			return AIRSPY_ERROR_NO_MEM;
 		}
 
-		for(transfer_index=0; transfer_index<device->transfer_count; transfer_index++)
+		for (transfer_index = 0; transfer_index<device->transfer_count; transfer_index++)
 		{
 			device->transfers[transfer_index] = libusb_alloc_transfer(0);
-			if( device->transfers[transfer_index] == NULL )
+			if (device->transfers[transfer_index] == NULL)
 			{
 				return AIRSPY_ERROR_LIBUSB;
 			}
 
 			libusb_fill_bulk_transfer(
-			device->transfers[transfer_index],
-			device->usb_device,
-			0,
-			(unsigned char*)malloc(device->buffer_size),
-			device->buffer_size,
-			NULL,
-			device,
-			0
-			);
+				device->transfers[transfer_index],
+				device->usb_device,
+				0,
+				(unsigned char*)malloc(device->buffer_size),
+				device->buffer_size,
+				NULL,
+				device,
+				0
+				);
 
-			if( device->transfers[transfer_index]->buffer == NULL )
+			if (device->transfers[transfer_index]->buffer == NULL)
 			{
 				return AIRSPY_ERROR_NO_MEM;
 			}
@@ -263,21 +271,22 @@ static int prepare_transfers(airspy_device_t* device, const uint_fast8_t endpoin
 {
 	int error;
 	uint32_t transfer_index;
-	if( device->transfers != NULL )
+	if (device->transfers != NULL)
 	{
-		for(transfer_index=0; transfer_index<device->transfer_count; transfer_index++)
+		for (transfer_index = 0; transfer_index<device->transfer_count; transfer_index++)
 		{
 			device->transfers[transfer_index]->endpoint = endpoint_address;
 			device->transfers[transfer_index]->callback = callback;
 
 			error = libusb_submit_transfer(device->transfers[transfer_index]);
-			if( error != 0 )
+			if (error != 0)
 			{
 				return AIRSPY_ERROR_LIBUSB;
 			}
 		}
 		return AIRSPY_SUCCESS;
-	} else {
+	}
+	else {
 		// This shouldn't happen.
 		return AIRSPY_ERROR_OTHER;
 	}
@@ -318,10 +327,11 @@ static inline void unpack_samples(uint32_t *input, uint16_t *output, int length)
 	}
 }
 
-static void* conversion_threadproc(void *arg)
+static void* consumer_threadproc(void *arg)
 {
 	int sample_count;
 	uint16_t* input_samples;
+	uint32_t dropped_buffers;
 	airspy_device_t* device = (airspy_device_t*)arg;
 	airspy_transfer_t transfer;
 
@@ -331,35 +341,35 @@ static void* conversion_threadproc(void *arg)
 
 #endif
 
+	pthread_mutex_lock(&device->consumer_mp);
+
 	while (device->streaming && !device->stop_requested)
 	{
-		if (device->received_samples_queue_head == device->received_samples_queue_tail)
+		while (device->received_buffer_count == 0 && device->streaming && !device->stop_requested)
 		{
-			pthread_mutex_lock(&device->conversion_mp);
-			device->converter_is_waiting = true;
-			while (device->received_samples_queue_head == device->received_samples_queue_tail &&
-				!device->stop_requested && device->streaming)
-			{
-				pthread_cond_wait(&device->conversion_cv, &device->conversion_mp);
-			}
-			device->converter_is_waiting = false;
-			pthread_mutex_unlock(&device->conversion_mp);
-
-			if (device->stop_requested || !device->streaming)
-			{
-				continue;
-			}
+			pthread_cond_wait(&device->consumer_cv, &device->consumer_mp);
+		}
+		if (!device->streaming || device->stop_requested)
+		{
+			break;
 		}
 
 		input_samples = device->received_samples_queue[device->received_samples_queue_tail];
-		
+		dropped_buffers = device->dropped_buffers_queue[device->received_samples_queue_tail];
+		device->received_samples_queue_tail = (device->received_samples_queue_tail + 1) & (RAW_BUFFER_COUNT - 1);
+
+		pthread_mutex_unlock(&device->consumer_mp);
+
 		if (device->packing_enabled)
 		{
 			sample_count = ((device->buffer_size / 2) * 4) / 3;
 
-			unpack_samples((uint32_t*)input_samples, device->unpacked_samples, sample_count);
-			
-			input_samples = device->unpacked_samples;
+			if (device->sample_type != AIRSPY_SAMPLE_RAW)
+			{
+				unpack_samples((uint32_t*)input_samples, device->unpacked_samples, sample_count);
+
+				input_samples = device->unpacked_samples;
+			}
 		}
 		else
 		{
@@ -393,6 +403,7 @@ static void* conversion_threadproc(void *arg)
 			break;
 
 		case AIRSPY_SAMPLE_UINT16_REAL:
+		case AIRSPY_SAMPLE_RAW:
 			transfer.samples = input_samples;
 			break;
 
@@ -405,47 +416,65 @@ static void* conversion_threadproc(void *arg)
 		transfer.ctx = device->ctx;
 		transfer.sample_count = sample_count;
 		transfer.sample_type = device->sample_type;
+		transfer.dropped_samples = (uint64_t) dropped_buffers * (uint64_t) sample_count;
 
 		if (device->callback(&transfer) != 0)
 		{
 			device->stop_requested = true;
 		}
 
-		device->received_samples_queue_tail = (device->received_samples_queue_tail + 1) & (RAW_BUFFER_COUNT - 1);
+		pthread_mutex_lock(&device->consumer_mp);
+		device->received_buffer_count--;
 	}
 
+	pthread_mutex_unlock(&device->consumer_mp);
+
+	pthread_exit(NULL);
+
 	return NULL;
 }
 
 static void airspy_libusb_transfer_callback(struct libusb_transfer* usb_transfer)
 {
 	uint16_t *temp;
-	airspy_device_t* device = (airspy_device_t*) usb_transfer->user_data;
+	airspy_device_t* device = (airspy_device_t*)usb_transfer->user_data;
 
 	if (!device->streaming || device->stop_requested)
 	{
 		return;
 	}
 
-	if (usb_transfer->status == LIBUSB_TRANSFER_COMPLETED)
+	if (usb_transfer->status == LIBUSB_TRANSFER_COMPLETED && usb_transfer->actual_length == usb_transfer->length)
 	{
-		if (device->received_samples_queue_head != device->received_samples_queue_tail || device->converter_is_waiting)
+		pthread_mutex_lock(&device->consumer_mp);
+
+		if (device->received_buffer_count < RAW_BUFFER_COUNT)
 		{
 			temp = device->received_samples_queue[device->received_samples_queue_head];
-			device->received_samples_queue[device->received_samples_queue_head] = (uint16_t *) usb_transfer->buffer;
-			usb_transfer->buffer = (uint8_t *) temp;
-			device->received_samples_queue_head = (device->received_samples_queue_head + 1) & (RAW_BUFFER_COUNT - 1);
+			device->received_samples_queue[device->received_samples_queue_head] = (uint16_t *)usb_transfer->buffer;
+			usb_transfer->buffer = (uint8_t *)temp;
 
-			if (device->converter_is_waiting)
-			{
-				pthread_mutex_lock(&device->conversion_mp);
-				pthread_cond_signal(&device->conversion_cv);
-				pthread_mutex_unlock(&device->conversion_mp);
-			}
+			device->dropped_buffers_queue[device->received_samples_queue_head] = device->dropped_buffers;
+			device->dropped_buffers = 0;
+			
+			device->received_samples_queue_head = (device->received_samples_queue_head + 1) & (RAW_BUFFER_COUNT - 1);
+			device->received_buffer_count++;
+
+			pthread_cond_signal(&device->consumer_cv);
+		}
+		else
+		{
+			device->dropped_buffers++;
+		}
+
+		pthread_mutex_unlock(&device->consumer_mp);
+
+		if (libusb_submit_transfer(usb_transfer) != 0)
+		{
+			device->streaming = false;
 		}
 	}
-
-	if (libusb_submit_transfer(usb_transfer) != 0)
+	else
 	{
 		device->streaming = false;
 	}
@@ -469,10 +498,12 @@ static void* transfer_threadproc(void* arg)
 		if (error < 0)
 		{
 			if (error != LIBUSB_ERROR_INTERRUPTED)
-			device->streaming = false;
+				device->streaming = false;
 		}
 	}
 
+	pthread_exit(NULL);
+
 	return NULL;
 }
 
@@ -483,10 +514,12 @@ static int kill_io_threads(airspy_device_t* device)
 		device->stop_requested = true;
 		cancel_transfers(device);
 
-		pthread_cond_signal(&device->conversion_cv);
+		pthread_mutex_lock(&device->consumer_mp);
+		pthread_cond_signal(&device->consumer_cv);
+		pthread_mutex_unlock(&device->consumer_mp);
 
 		pthread_join(device->transfer_thread, NULL);
-		pthread_join(device->conversion_thread, NULL);
+		pthread_join(device->consumer_thread, NULL);
 
 		device->stop_requested = false;
 		device->streaming = false;
@@ -505,7 +538,7 @@ static int create_io_threads(airspy_device_t* device, airspy_sample_block_cb_fn
 		device->callback = callback;
 		device->streaming = true;
 
-		result = prepare_transfers(device, LIBUSB_ENDPOINT_IN | 1, (libusb_transfer_cb_fn) airspy_libusb_transfer_callback);
+		result = prepare_transfers(device, LIBUSB_ENDPOINT_IN | 1, (libusb_transfer_cb_fn)airspy_libusb_transfer_callback);
 		if (result != AIRSPY_SUCCESS)
 		{
 			return result;
@@ -513,12 +546,12 @@ static int create_io_threads(airspy_device_t* device, airspy_sample_block_cb_fn
 
 		device->received_samples_queue_head = 0;
 		device->received_samples_queue_tail = 0;
-		device->converter_is_waiting = true;
+		device->received_buffer_count = 0;
 
 		pthread_attr_init(&attr);
 		pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);
 
-		result = pthread_create(&device->conversion_thread, &attr, conversion_threadproc, device);
+		result = pthread_create(&device->consumer_thread, &attr, consumer_threadproc, device);
 		if (result != 0)
 		{
 			return AIRSPY_ERROR_THREAD;
@@ -541,7 +574,7 @@ static int create_io_threads(airspy_device_t* device, airspy_sample_block_cb_fn
 
 static void airspy_open_exit(airspy_device_t* device)
 {
-	if(device->usb_device != NULL)
+	if (device->usb_device != NULL)
 	{
 		libusb_release_interface(device->usb_device, 0);
 		libusb_close(device->usb_device);
@@ -565,10 +598,10 @@ static void upper_string(unsigned char *string, size_t len)
 }
 
 static void airspy_open_device(airspy_device_t* device,
-								int* ret,
-								uint16_t vid,
-								uint16_t pid,
-								uint64_t serial_number_val)
+	int* ret,
+	uint16_t vid,
+	uint16_t pid,
+	uint64_t serial_number_val)
 {
 	int i;
 	int result;
@@ -581,8 +614,8 @@ static void airspy_open_device(airspy_device_t* device,
 	ssize_t cnt;
 	int serial_descriptor_index;
 	struct libusb_device_descriptor device_descriptor;
-	char serial_number_expected[SERIAL_AIRSPY_EXPECTED_SIZE+1];
-	unsigned char serial_number[SERIAL_AIRSPY_EXPECTED_SIZE+1];
+	char serial_number_expected[SERIAL_AIRSPY_EXPECTED_SIZE + 1];
+	unsigned char serial_number[SERIAL_AIRSPY_EXPECTED_SIZE + 1];
 	uint32_t serial_number_msb_val;
 	uint32_t serial_number_lsb_val;
 
@@ -597,7 +630,7 @@ static void airspy_open_device(airspy_device_t* device,
 	}
 
 	i = 0;
-	while( (dev = devices[i++]) != NULL )
+	while ((dev = devices[i++]) != NULL)
 	{
 		libusb_get_device_descriptor(dev, &device_descriptor);
 
@@ -616,9 +649,9 @@ static void airspy_open_device(airspy_device_t* device,
 					}
 					dev_handle = *libusb_dev_handle;
 					serial_number_len = libusb_get_string_descriptor_ascii(dev_handle,
-																			  serial_descriptor_index,
-																			  serial_number,
-																			  sizeof(serial_number));
+						serial_descriptor_index,
+						serial_number,
+						sizeof(serial_number));
 					if (serial_number_len == SERIAL_AIRSPY_EXPECTED_SIZE)
 					{
 						serial_number[SERIAL_AIRSPY_EXPECTED_SIZE] = 0;
@@ -627,16 +660,16 @@ static void airspy_open_device(airspy_device_t* device,
 						serial_number_lsb_val = (uint32_t)(serial_number_val & 0xFFFFFFFF);
 
 						sprintf(serial_number_expected, "%s%08X%08X",
-								str_prefix_serial_airspy,
-								serial_number_msb_val,
-								serial_number_lsb_val);
+							str_prefix_serial_airspy,
+							serial_number_msb_val,
+							serial_number_lsb_val);
 
 						if (strncmp((const char*)serial_number, serial_number_expected, SERIAL_AIRSPY_EXPECTED_SIZE) == 0)
 						{
 #ifdef __linux__
-							/* Check whether a kernel driver is attached to interface #0. If so, we'll 
-							 * need to detach it.
-							 */
+							/* Check whether a kernel driver is attached to interface #0. If so, we'll
+							* need to detach it.
+							*/
 							if (libusb_kernel_driver_active(dev_handle, 0))
 							{
 								libusb_detach_kernel_driver(dev_handle, 0);
@@ -657,28 +690,31 @@ static void airspy_open_device(airspy_device_t* device,
 								continue;
 							}
 							break;
-						} else
+						}
+						else
 						{
 							libusb_close(dev_handle);
 							*libusb_dev_handle = NULL;
 							continue;
 						}
-					}else 
+					}
+					else
 					{
 						libusb_close(dev_handle);
 						*libusb_dev_handle = NULL;
 						continue;
 					}
 				}
-			}else 
+			}
+			else
 			{
 				if (libusb_open(dev, libusb_dev_handle) == 0)
 				{
 					dev_handle = *libusb_dev_handle;
 #ifdef __linux__
-					/* Check whether a kernel driver is attached to interface #0. If so, we'll 
-					 * need to detach it.
-					 */
+					/* Check whether a kernel driver is attached to interface #0. If so, we'll
+					* need to detach it.
+					*/
 					if (libusb_kernel_driver_active(dev_handle, 0))
 					{
 						libusb_detach_kernel_driver(dev_handle, 0);
@@ -716,6 +752,28 @@ static void airspy_open_device(airspy_device_t* device,
 	return;
 }
 
+static int airspy_read_samplerates_from_fw(struct airspy_device* device, uint32_t* buffer, const uint32_t len)
+{
+	int result;
+
+	result = libusb_control_transfer(
+		device->usb_device,
+		LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+		AIRSPY_GET_SAMPLERATES,
+		0,
+		len,
+		(unsigned char*)buffer,
+		(len > 0 ? len : 1) * sizeof(uint32_t),
+		0);
+
+	if (result < 1)
+	{
+		return AIRSPY_ERROR_OTHER;
+	}
+
+	return AIRSPY_SUCCESS;
+}
+
 static int airspy_open_init(airspy_device_t** device, uint64_t serial_number)
 {
 	airspy_device_t* lib_device;
@@ -725,24 +783,24 @@ static int airspy_open_init(airspy_device_t** device, uint64_t serial_number)
 	*device = NULL;
 
 	lib_device = (airspy_device_t*)calloc(1, sizeof(airspy_device_t));
-	if(lib_device == NULL)
+	if (lib_device == NULL)
 	{
 		return AIRSPY_ERROR_NO_MEM;
 	}
 
 	libusb_error = libusb_init(&lib_device->usb_context);
-	if(libusb_error != 0)
+	if (libusb_error != 0)
 	{
 		free(lib_device);
 		return AIRSPY_ERROR_LIBUSB;
 	}
 
 	airspy_open_device(lib_device,
-										&result,
-										airspy_usb_vid,
-										airspy_usb_pid,
-										serial_number);
-	if(lib_device->usb_device == NULL)
+		&result,
+		airspy_usb_vid,
+		airspy_usb_pid,
+		serial_number);
+	if (lib_device->usb_device == NULL)
 	{
 		libusb_exit(lib_device->usb_context);
 		free(lib_device);
@@ -758,12 +816,32 @@ static int airspy_open_init(airspy_device_t** device, uint64_t serial_number)
 	lib_device->stop_requested = false;
 	lib_device->sample_type = AIRSPY_SAMPLE_FLOAT32_IQ;
 
+	result = airspy_read_samplerates_from_fw(lib_device, &lib_device->supported_samplerate_count, 0);
+	if (result == AIRSPY_SUCCESS)
+	{
+		lib_device->supported_samplerates = (uint32_t *) malloc(lib_device->supported_samplerate_count * sizeof(uint32_t));
+		result = airspy_read_samplerates_from_fw(lib_device, lib_device->supported_samplerates, lib_device->supported_samplerate_count);
+		if (result != AIRSPY_SUCCESS)
+		{
+			free(lib_device->supported_samplerates);
+		}
+	}
+
+	if (result != AIRSPY_SUCCESS)
+	{
+		lib_device->supported_samplerate_count = 2;
+		lib_device->supported_samplerates = (uint32_t *) malloc(lib_device->supported_samplerate_count * sizeof(uint32_t));
+		lib_device->supported_samplerates[0] = 10000000;
+		lib_device->supported_samplerates[1] = 2500000;
+	}
+
 	airspy_set_packing(lib_device, 0);
 
 	result = allocate_transfers(lib_device);
-	if( result != 0 )
+	if (result != 0)
 	{
 		airspy_open_exit(lib_device);
+		free(lib_device->supported_samplerates);
 		free(lib_device);
 		return AIRSPY_ERROR_NO_MEM;
 	}
@@ -771,8 +849,8 @@ static int airspy_open_init(airspy_device_t** device, uint64_t serial_number)
 	lib_device->cnv_f = iqconverter_float_create(HB_KERNEL_FLOAT, HB_KERNEL_FLOAT_LEN);
 	lib_device->cnv_i = iqconverter_int16_create(HB_KERNEL_INT16, HB_KERNEL_INT16_LEN);
 
-	pthread_cond_init(&lib_device->conversion_cv, NULL);
-	pthread_mutex_init(&lib_device->conversion_mp, NULL);
+	pthread_cond_init(&lib_device->consumer_cv, NULL);
+	pthread_mutex_init(&lib_device->consumer_mp, NULL);
 
 	*device = lib_device;
 
@@ -824,19 +902,20 @@ extern "C"
 		int result;
 
 		result = AIRSPY_SUCCESS;
-		
-		if(device != NULL)
+
+		if (device != NULL)
 		{
 			result = airspy_stop_rx(device);
 
 			iqconverter_float_free(device->cnv_f);
 			iqconverter_int16_free(device->cnv_i);
 
-			pthread_cond_destroy(&device->conversion_cv);
-			pthread_mutex_destroy(&device->conversion_mp);
+			pthread_cond_destroy(&device->consumer_cv);
+			pthread_mutex_destroy(&device->consumer_mp);
 
 			airspy_open_exit(device);
 			free_transfers(device);
+			free(device->supported_samplerates);
 			free(device);
 		}
 
@@ -845,34 +924,29 @@ extern "C"
 
 	int ADDCALL airspy_get_samplerates(struct airspy_device* device, uint32_t* buffer, const uint32_t len)
 	{
-		int result;
+		uint32_t i;
 
-		result = libusb_control_transfer(
-			device->usb_device,
-			LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
-			AIRSPY_GET_SAMPLERATES,
-			0,
-			len,
-			(unsigned char*) buffer,
-			(len > 0 ? len : 1) * sizeof(uint32_t),
-			0);
-
-		if (result < 1)
+		if (len == 0)
 		{
-			if (len == 0)
+			*buffer = device->supported_samplerate_count;
+		}
+		else if (len <= device->supported_samplerate_count)
+		{
+			memcpy(buffer, device->supported_samplerates, len * sizeof(uint32_t));
+
+			if (!SAMPLE_TYPE_IS_IQ(device->sample_type))
 			{
-				*buffer = 2;
-			}
-			else if (len >= 2)
-			{
-				buffer[0] = 10000000;
-				buffer[1] = 2500000;
-			}
-			else
-			{
-				return AIRSPY_ERROR_INVALID_PARAM;
+				for (i = 0; i < len; i++)
+				{
+					buffer[i] *= 2;
+				}
 			}
 		}
+		else
+		{
+			return AIRSPY_ERROR_INVALID_PARAM;
+		}
+
 		return AIRSPY_SUCCESS;
 	}
 
@@ -881,26 +955,49 @@ extern "C"
 		int result;
 		uint8_t retval;
 		uint8_t length;
+		uint32_t i;
+
+		if (samplerate >= MIN_SAMPLERATE_BY_VALUE)
+		{
+			for (i = 0; i < device->supported_samplerate_count; i++)
+			{
+				if (samplerate == device->supported_samplerates[i])
+				{
+					samplerate = i;
+					break;
+				}
+			}
+
+			if (samplerate >= MIN_SAMPLERATE_BY_VALUE)
+			{
+				if (SAMPLE_TYPE_IS_IQ(device->sample_type))
+				{
+					samplerate *= 2;
+				}
+				samplerate /= 1000;
+			}
+		}
 
 		libusb_clear_halt(device->usb_device, LIBUSB_ENDPOINT_IN | 1);
 
 		length = 1;
 
 		result = libusb_control_transfer(
-		device->usb_device,
-		LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
-		AIRSPY_SET_SAMPLERATE,
-		0,
-		samplerate,
-		&retval,
-		length,
-		0
-		);
+			device->usb_device,
+			LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+			AIRSPY_SET_SAMPLERATE,
+			0,
+			samplerate,
+			&retval,
+			length,
+			0
+			);
 
 		if (result < length)
 		{
 			return AIRSPY_ERROR_LIBUSB;
-		} else {
+		}
+		else {
 			return AIRSPY_SUCCESS;
 		}
 	}
@@ -909,20 +1006,21 @@ extern "C"
 	{
 		int result;
 		result = libusb_control_transfer(
-		device->usb_device,
-		LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
-		AIRSPY_RECEIVER_MODE,
-		value,
-		0,
-		NULL,
-		0,
-		0
-		);
+			device->usb_device,
+			LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+			AIRSPY_RECEIVER_MODE,
+			value,
+			0,
+			NULL,
+			0,
+			0
+			);
 
-		if( result != 0 )
+		if (result != 0)
 		{
 			return AIRSPY_ERROR_LIBUSB;
-		} else {
+		}
+		else {
 			return AIRSPY_SUCCESS;
 		}
 	}
@@ -931,17 +1029,27 @@ extern "C"
 	{
 		int result;
 
-		libusb_clear_halt(device->usb_device, LIBUSB_ENDPOINT_IN | 1);
-
 		iqconverter_float_reset(device->cnv_f);
 		iqconverter_int16_reset(device->cnv_i);
-		
+
+		memset(device->dropped_buffers_queue, 0, RAW_BUFFER_COUNT * sizeof(uint32_t));
+		device->dropped_buffers = 0;
+
+		result = airspy_set_receiver_mode(device, RECEIVER_MODE_OFF);
+		if (result != AIRSPY_SUCCESS)
+		{
+			return result;
+		}
+
+		libusb_clear_halt(device->usb_device, LIBUSB_ENDPOINT_IN | 1);
+
 		result = airspy_set_receiver_mode(device, RECEIVER_MODE_RX);
-		if( result == AIRSPY_SUCCESS )
+		if (result == AIRSPY_SUCCESS)
 		{
 			device->ctx = ctx;
 			result = create_io_threads(device, callback);
 		}
+
 		return result;
 	}
 
@@ -965,19 +1073,20 @@ extern "C"
 
 		temp_value = 0;
 		result = libusb_control_transfer(
-		device->usb_device,
-		LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
-		AIRSPY_SI5351C_READ,
-		0,
-		register_number,
-		(unsigned char*)&temp_value,
-		1,
-		0);
+			device->usb_device,
+			LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+			AIRSPY_SI5351C_READ,
+			0,
+			register_number,
+			(unsigned char*)&temp_value,
+			1,
+			0);
 
-		if( result < 1 )
+		if (result < 1)
 		{
 			return AIRSPY_ERROR_LIBUSB;
-		} else {
+		}
+		else {
 			*value = temp_value;
 			return AIRSPY_SUCCESS;
 		}
@@ -988,19 +1097,20 @@ extern "C"
 		int result;
 
 		result = libusb_control_transfer(
-		device->usb_device,
-		LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
-		AIRSPY_SI5351C_WRITE,
-		value,
-		register_number,
-		NULL,
-		0,
-		0);
+			device->usb_device,
+			LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+			AIRSPY_SI5351C_WRITE,
+			value,
+			register_number,
+			NULL,
+			0,
+			0);
 
-		if( result != 0 )
+		if (result != 0)
 		{
 			return AIRSPY_ERROR_LIBUSB;
-		} else {
+		}
+		else {
 			return AIRSPY_SUCCESS;
 		}
 	}
@@ -1010,19 +1120,20 @@ extern "C"
 		int result;
 
 		result = libusb_control_transfer(
-		device->usb_device,
-		LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
-		AIRSPY_R820T_READ,
-		0,
-		register_number,
-		(unsigned char*) value,
-		1,
-		0);
+			device->usb_device,
+			LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+			AIRSPY_R820T_READ,
+			0,
+			register_number,
+			(unsigned char*)value,
+			1,
+			0);
 
-		if( result < 1 )
+		if (result < 1)
 		{
 			return AIRSPY_ERROR_LIBUSB;
-		} else {
+		}
+		else {
 			return AIRSPY_SUCCESS;
 		}
 	}
@@ -1030,21 +1141,22 @@ extern "C"
 	int ADDCALL airspy_r820t_write(airspy_device_t* device, uint8_t register_number, uint8_t value)
 	{
 		int result;
-		
-		result = libusb_control_transfer(
-		device->usb_device,
-		LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
-		AIRSPY_R820T_WRITE,
-		value,
-		register_number,
-		NULL,
-		0,
-		0);
 
-		if( result != 0 )
+		result = libusb_control_transfer(
+			device->usb_device,
+			LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+			AIRSPY_R820T_WRITE,
+			value,
+			register_number,
+			NULL,
+			0,
+			0);
+
+		if (result != 0)
 		{
 			return AIRSPY_ERROR_LIBUSB;
-		} else {
+		}
+		else {
 			return AIRSPY_SUCCESS;
 		}
 	}
@@ -1058,19 +1170,20 @@ extern "C"
 		port_pin = port_pin | pin;
 
 		result = libusb_control_transfer(
-		device->usb_device,
-		LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
-		AIRSPY_GPIO_READ,
-		0,
-		port_pin,
-		(unsigned char*) value,
-		1,
-		0);
+			device->usb_device,
+			LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+			AIRSPY_GPIO_READ,
+			0,
+			port_pin,
+			(unsigned char*)value,
+			1,
+			0);
 
-		if( result < 1 )
+		if (result < 1)
 		{
 			return AIRSPY_ERROR_LIBUSB;
-		} else {
+		}
+		else {
 			return AIRSPY_SUCCESS;
 		}
 	}
@@ -1084,19 +1197,20 @@ extern "C"
 		port_pin = port_pin | pin;
 
 		result = libusb_control_transfer(
-		device->usb_device,
-		LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
-		AIRSPY_GPIO_WRITE,
-		value,
-		port_pin,
-		NULL,
-		0,
-		0);
+			device->usb_device,
+			LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+			AIRSPY_GPIO_WRITE,
+			value,
+			port_pin,
+			NULL,
+			0,
+			0);
 
-		if( result != 0 )
+		if (result != 0)
 		{
 			return AIRSPY_ERROR_LIBUSB;
-		} else {
+		}
+		else {
 			return AIRSPY_SUCCESS;
 		}
 	}
@@ -1111,19 +1225,20 @@ extern "C"
 		port_pin = port_pin | pin;
 
 		result = libusb_control_transfer(
-		device->usb_device,
-		LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
-		AIRSPY_GPIODIR_READ,
-		0,
-		port_pin,
-		(unsigned char*) value,
-		1,
-		0);
+			device->usb_device,
+			LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+			AIRSPY_GPIODIR_READ,
+			0,
+			port_pin,
+			(unsigned char*)value,
+			1,
+			0);
 
-		if( result < 1 )
+		if (result < 1)
 		{
 			return AIRSPY_ERROR_LIBUSB;
-		} else {
+		}
+		else {
 			return AIRSPY_SUCCESS;
 		}
 	}
@@ -1137,19 +1252,20 @@ extern "C"
 		port_pin = port_pin | pin;
 
 		result = libusb_control_transfer(
-		device->usb_device,
-		LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
-		AIRSPY_GPIODIR_WRITE,
-		value,
-		port_pin,
-		NULL,
-		0,
-		0);
+			device->usb_device,
+			LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+			AIRSPY_GPIODIR_WRITE,
+			value,
+			port_pin,
+			NULL,
+			0,
+			0);
 
-		if( result != 0 )
+		if (result != 0)
 		{
 			return AIRSPY_ERROR_LIBUSB;
-		} else {
+		}
+		else {
 			return AIRSPY_SUCCESS;
 		}
 	}
@@ -1158,19 +1274,20 @@ extern "C"
 	{
 		int result;
 		result = libusb_control_transfer(
-		device->usb_device,
-		LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
-		AIRSPY_SPIFLASH_ERASE,
-		0,
-		0,
-		NULL,
-		0,
-		0);
+			device->usb_device,
+			LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+			AIRSPY_SPIFLASH_ERASE,
+			0,
+			0,
+			NULL,
+			0,
+			0);
 
 		if (result != 0)
 		{
 			return AIRSPY_ERROR_LIBUSB;
-		} else {
+		}
+		else {
 			return AIRSPY_SUCCESS;
 		}
 	}
@@ -1178,26 +1295,27 @@ extern "C"
 	int ADDCALL airspy_spiflash_write(airspy_device_t* device, const uint32_t address, const uint16_t length, unsigned char* const data)
 	{
 		int result;
-		
+
 		if (address > 0x0FFFFF)
 		{
 			return AIRSPY_ERROR_INVALID_PARAM;
 		}
 
 		result = libusb_control_transfer(
-		device->usb_device,
-		LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
-		AIRSPY_SPIFLASH_WRITE,
-		address >> 16,
-		address & 0xFFFF,
-		data,
-		length,
-		0);
+			device->usb_device,
+			LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+			AIRSPY_SPIFLASH_WRITE,
+			address >> 16,
+			address & 0xFFFF,
+			data,
+			length,
+			0);
 
 		if (result < length)
 		{
 			return AIRSPY_ERROR_LIBUSB;
-		} else {
+		}
+		else {
 			return AIRSPY_SUCCESS;
 		}
 	}
@@ -1207,19 +1325,20 @@ extern "C"
 		int result;
 
 		result = libusb_control_transfer(
-		device->usb_device,
-		LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
-		AIRSPY_SPIFLASH_READ,
-		address >> 16,
-		address & 0xFFFF,
-		data,
-		length,
-		0);
+			device->usb_device,
+			LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+			AIRSPY_SPIFLASH_READ,
+			address >> 16,
+			address & 0xFFFF,
+			data,
+			length,
+			0);
 
 		if (result < length)
 		{
 			return AIRSPY_ERROR_LIBUSB;
-		} else {
+		}
+		else {
 			return AIRSPY_SUCCESS;
 		}
 	}
@@ -1228,48 +1347,50 @@ extern "C"
 	{
 		int result;
 		result = libusb_control_transfer(
-		device->usb_device,
-		LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
-		AIRSPY_BOARD_ID_READ,
-		0,
-		0,
-		value,
-		1,
-		0);
+			device->usb_device,
+			LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+			AIRSPY_BOARD_ID_READ,
+			0,
+			0,
+			value,
+			1,
+			0);
 
 		if (result < 1)
 		{
 			return AIRSPY_ERROR_LIBUSB;
-		} else {
+		}
+		else {
 			return AIRSPY_SUCCESS;
 		}
 	}
 
 	int ADDCALL airspy_version_string_read(airspy_device_t* device, char* version, uint8_t length)
 	{
-		#define VERSION_LOCAL_SIZE (128)
+#define VERSION_LOCAL_SIZE (128)
 		int result;
 		char version_local[VERSION_LOCAL_SIZE];
 
 		result = libusb_control_transfer(
-		device->usb_device,
-		LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
-		AIRSPY_VERSION_STRING_READ,
-		0,
-		0,
-		(unsigned char*)version_local,
-		(VERSION_LOCAL_SIZE-1),
-		0);
+			device->usb_device,
+			LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+			AIRSPY_VERSION_STRING_READ,
+			0,
+			0,
+			(unsigned char*)version_local,
+			(VERSION_LOCAL_SIZE - 1),
+			0);
 
 		if (result < 0)
 		{
 			return AIRSPY_ERROR_LIBUSB;
-		} else
+		}
+		else
 		{
-			if(length > 0)
+			if (length > 0)
 			{
-				memcpy(version, version_local, length-1);
-				version[length-1] = 0;
+				memcpy(version, version_local, length - 1);
+				version[length - 1] = 0;
 				return AIRSPY_SUCCESS;
 			}
 			else
@@ -1283,22 +1404,23 @@ extern "C"
 	{
 		uint8_t length;
 		int result;
-		
+
 		length = sizeof(airspy_read_partid_serialno_t);
 		result = libusb_control_transfer(
-		device->usb_device,
-		LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
-		AIRSPY_BOARD_PARTID_SERIALNO_READ,
-		0,
-		0,
-		(unsigned char*)read_partid_serialno,
-		length,
-		0);
+			device->usb_device,
+			LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+			AIRSPY_BOARD_PARTID_SERIALNO_READ,
+			0,
+			0,
+			(unsigned char*)read_partid_serialno,
+			length,
+			0);
 
 		if (result < length)
 		{
 			return AIRSPY_ERROR_LIBUSB;
-		} else {
+		}
+		else {
 
 			read_partid_serialno->part_id[0] = TO_LE(read_partid_serialno->part_id[0]);
 			read_partid_serialno->part_id[1] = TO_LE(read_partid_serialno->part_id[1]);
@@ -1327,20 +1449,21 @@ extern "C"
 		length = sizeof(set_freq_params_t);
 
 		result = libusb_control_transfer(
-		device->usb_device,
-		LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
-		AIRSPY_SET_FREQ,
-		0,
-		0,
-		(unsigned char*)&set_freq_params,
-		length,
-		0
-		);
+			device->usb_device,
+			LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+			AIRSPY_SET_FREQ,
+			0,
+			0,
+			(unsigned char*)&set_freq_params,
+			length,
+			0
+			);
 
 		if (result < length)
 		{
 			return AIRSPY_ERROR_LIBUSB;
-		} else {
+		}
+		else {
 			return AIRSPY_SUCCESS;
 		}
 	}
@@ -1383,20 +1506,21 @@ extern "C"
 		length = 1;
 
 		result = libusb_control_transfer(
-		device->usb_device,
-		LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
-		AIRSPY_SET_LNA_GAIN,
-		0,
-		value,
-		&retval,
-		length,
-		0
-		);
+			device->usb_device,
+			LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+			AIRSPY_SET_LNA_GAIN,
+			0,
+			value,
+			&retval,
+			length,
+			0
+			);
 
 		if (result < length)
 		{
 			return AIRSPY_ERROR_LIBUSB;
-		} else {
+		}
+		else {
 			return AIRSPY_SUCCESS;
 		}
 	}
@@ -1413,20 +1537,21 @@ extern "C"
 		length = 1;
 
 		result = libusb_control_transfer(
-		device->usb_device,
-		LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
-		AIRSPY_SET_MIXER_GAIN,
-		0,
-		value,
-		&retval,
-		length,
-		0
-		);
+			device->usb_device,
+			LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+			AIRSPY_SET_MIXER_GAIN,
+			0,
+			value,
+			&retval,
+			length,
+			0
+			);
 
 		if (result < length)
 		{
 			return AIRSPY_ERROR_LIBUSB;
-		} else {
+		}
+		else {
 			return AIRSPY_SUCCESS;
 		}
 	}
@@ -1443,20 +1568,21 @@ extern "C"
 		length = 1;
 
 		result = libusb_control_transfer(
-		device->usb_device,
-		LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
-		AIRSPY_SET_VGA_GAIN,
-		0,
-		value,
-		&retval,
-		length,
-		0
-		);
+			device->usb_device,
+			LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+			AIRSPY_SET_VGA_GAIN,
+			0,
+			value,
+			&retval,
+			length,
+			0
+			);
 
 		if (result < length)
 		{
 			return AIRSPY_ERROR_LIBUSB;
-		} else {
+		}
+		else {
 			return AIRSPY_SUCCESS;
 		}
 	}
@@ -1470,20 +1596,21 @@ extern "C"
 		length = 1;
 
 		result = libusb_control_transfer(
-		device->usb_device,
-		LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
-		AIRSPY_SET_LNA_AGC,
-		0,
-		value,
-		&retval,
-		length,
-		0
-		);
+			device->usb_device,
+			LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+			AIRSPY_SET_LNA_AGC,
+			0,
+			value,
+			&retval,
+			length,
+			0
+			);
 
 		if (result < length)
 		{
 			return AIRSPY_ERROR_LIBUSB;
-		} else {
+		}
+		else {
 			return AIRSPY_SUCCESS;
 		}
 	}
@@ -1497,20 +1624,21 @@ extern "C"
 		length = 1;
 
 		result = libusb_control_transfer(
-		device->usb_device,
-		LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
-		AIRSPY_SET_MIXER_AGC,
-		0,
-		value,
-		&retval,
-		length,
-		0
-		);
+			device->usb_device,
+			LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+			AIRSPY_SET_MIXER_AGC,
+			0,
+			value,
+			&retval,
+			length,
+			0
+			);
 
 		if (result < length)
 		{
 			return AIRSPY_ERROR_LIBUSB;
-		} else {
+		}
+		else {
 			return AIRSPY_SUCCESS;
 		}
 	}
@@ -1598,7 +1726,7 @@ extern "C"
 	{
 		int result;
 		uint8_t retval;
-		bool packing_enabled; 
+		bool packing_enabled;
 
 		if (device->streaming)
 		{
@@ -1606,37 +1734,37 @@ extern "C"
 		}
 
 		result = libusb_control_transfer(
-		device->usb_device,
-		LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
-		AIRSPY_SET_PACKING,
-		0,
-		value,
-		&retval,
-		1,
-		0);
+			device->usb_device,
+			LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+			AIRSPY_SET_PACKING,
+			0,
+			value,
+			&retval,
+			1,
+			0);
 
 		if (result < 1)
-		{			
+		{
 			return  AIRSPY_ERROR_LIBUSB;
 		}
-		
+
 		packing_enabled = value ? true : false;
 		if (packing_enabled != device->packing_enabled)
-		{			
+		{
 			cancel_transfers(device);
 			free_transfers(device);
 
 			device->packing_enabled = packing_enabled;
 			device->buffer_size = packing_enabled ? (6144 * 24) : 262144;
-			
+
 			result = allocate_transfers(device);
 			if (result != 0)
 			{
 				return AIRSPY_ERROR_NO_MEM;
 			}
 		}
-		
-		return AIRSPY_SUCCESS;	
+
+		return AIRSPY_SUCCESS;
 	}
 
 	int ADDCALL airspy_is_streaming(airspy_device_t* device)
@@ -1646,7 +1774,7 @@ extern "C"
 
 	const char* ADDCALL airspy_error_name(enum airspy_error errcode)
 	{
-		switch(errcode)
+		switch (errcode)
 		{
 		case AIRSPY_SUCCESS:
 			return "AIRSPY_SUCCESS";
@@ -1688,7 +1816,7 @@ extern "C"
 
 	const char* ADDCALL airspy_board_id_name(enum airspy_board_id board_id)
 	{
-		switch(board_id)
+		switch (board_id)
 		{
 		case AIRSPY_BOARD_ID_PROTO_AIRSPY:
 			return "AIRSPY";
