@@ -28,10 +28,10 @@ ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSI
 #include <stdint.h>
 #include "airspy_commands.h"
 
-#define AIRSPY_VERSION "1.0.7"
+#define AIRSPY_VERSION "1.0.8"
 #define AIRSPY_VER_MAJOR 1
 #define AIRSPY_VER_MINOR 0
-#define AIRSPY_VER_REVISION 7
+#define AIRSPY_VER_REVISION 8
 
 #ifdef _WIN32
 	 #define ADD_EXPORTS
@@ -87,7 +87,8 @@ enum airspy_sample_type
 	AIRSPY_SAMPLE_INT16_IQ = 2,     /* 2 * 16bit int per sample */
 	AIRSPY_SAMPLE_INT16_REAL = 3,   /* 1 * 16bit int per sample */
 	AIRSPY_SAMPLE_UINT16_REAL = 4,  /* 1 * 16bit unsigned int per sample */
-	AIRSPY_SAMPLE_END = 5           /* Number of supported sample types */
+	AIRSPY_SAMPLE_RAW = 5,          /* Raw packed samples from the device */
+	AIRSPY_SAMPLE_END = 6           /* Number of supported sample types */
 };
 
 struct airspy_device;
@@ -97,6 +98,7 @@ typedef struct {
 	void* ctx;
 	void* samples;
 	int sample_count;
+	uint64_t dropped_samples;
 	enum airspy_sample_type sample_type;
 } airspy_transfer_t, airspy_transfer;
 
@@ -124,6 +126,8 @@ extern ADDAPI int ADDCALL airspy_open(struct airspy_device** device);
 extern ADDAPI int ADDCALL airspy_close(struct airspy_device* device);
 
 extern ADDAPI int ADDCALL airspy_get_samplerates(struct airspy_device* device, uint32_t* buffer, const uint32_t len);
+
+/* Parameter samplerate can be either the index of a samplerate or directly its value in Hz within the list returned by airspy_get_samplerates() */
 extern ADDAPI int ADDCALL airspy_set_samplerate(struct airspy_device* device, uint32_t samplerate);
 
 extern ADDAPI int ADDCALL airspy_set_conversion_filter_float32(struct airspy_device* device, const float *kernel, const uint32_t len);
